%% Estimation of the generalized Pareto curve b(p) using income tax data
%
% Method: PCHIP interpolation with mobile points
%
% Inputs:
%   * PP: vector of points p (in [0,1]) where b(p) has to be calculated
%   * thr: vector of thresholds of the tax data
%   * pp: vector of percentiles (in [0,1]) corresponding to the thresholds in the tax
%   tabulation
%   * bb: vector of (inverted) Pareto coefficients corresponding to the
%   thresholds in the tax tabulation
%   * y_av: the average income in the population
%   * p0: the Dirac in 0
%
% Output:
%   * BB: table of approximative values taken by b at points of the input
%   table PP, that is, BB=b(PP)
%
% Steps of the program:
%   1. Function pc
%     1.a. Preliminaries
%     1.b. Approximation of the upper part (above the first threshold)
%     1.c. Approximation above the last threshold
%     1.d. Approximation of the lower part
%   2. Function f_beta
%

%% 1. Function pc

function [BB] = pc2(PP,thr,pp,bb,y_av,p0)

%% 1.a. Preliminaries
% Defines function inputs and outputs.

% output vector BB
BB = ones(size(PP));

% number of tax thresholds
T = length(pp);

% vector beta
beta = log(((1-pp(1:(end-1))).*thr(1:(end-1)).*bb(1:(end-1)))./((1-pp(2:end)).*thr(2:end).*bb(2:end)));

% beta0, value of the integral between 0 (or p0) and pp(1)
beta0 = log(y_av/((1-pp(1))*thr(1)*bb(1)));

% minimal positive income
thr_min = min(y_av/1000,thr(1)/100);

% epsilon determining the precision of the approximation of the beta(i)
eps = 1e-14;

% time limit for while loops
timelimit = 20;

%% 1.b. Approximation of the upper part (above the first threshold)

% vectors of all points that have been set so far
Lp = []; % percentiles
Lb = []; % corresponding b

% browse through the raw brackets to set the intermediary points so that
% the successive integrals of 1/((1-p)b(p)) are equals to the betas

for i=1:T-1
    disp(['Currently estimating the Pareto curve within bracket ',num2str(i),'...'])
    
    if (i>1)&&(i<T-1)
        % points of interest in the loop: p2 and p3 delimit the bracket where
        % the mobile points lie, p1 and p4 are the surronding percentiles
        p1 = pp(i-1);
        p2 = pp(i);
        p3 = pp(i+1);
        p4 = pp(i+2);
        
        % corresponding Pareto coefficients
        b1 = bb(i-1);
        b2 = bb(i);
        b3 = bb(i+1);
        b4 = bb(i+2);
    elseif (i==1)&&(i<T-1)
        if isnan(p0)
            p1 = 0;
        else
            p1 = p0;
        end
        p2 = pp(i);
        p3 = pp(i+1);
        p4 = pp(i+2);
        
        % corresponding Pareto coefficients
        if isnan(p0)
            b1 = y_av/thr_min;
        else
            b1 = y_av/((1-p0)*thr_min);
        end
        b2 = bb(i);
        b3 = bb(i+1);
        b4 = bb(i+2);
    elseif (i>1)&&(i==T-1)
        p1 = pp(i-1);
        p2 = pp(i);
        p3 = pp(i+1);
        p4 = 1;
        
        % corresponding Pareto coefficients
        b1 = bb(i-1);
        b2 = bb(i);
        b3 = bb(i+1);
        bf = max(interp1([pp(end-1); pp(end)],[bb(end-1); bb(end)],p4,'linear','extrap'),1);
        b4 = bf;
    else % T=2
        if isnan(p0)
            p1 = 0;
        else
            p1 = p0;
        end
        p2 = pp(i);
        p3 = pp(i+1);
        p4 = 1;
        
        % corresponding Pareto coefficients
        if isnan(p0)
            b1 = y_av/thr_min;
        else
            b1 = y_av/((1-p0)*thr_min);
        end
        b2 = bb(i);
        b3 = bb(i+1);
        bf = max(interp1([pp(end-1); pp(end)],[bb(end-1); bb(end)],p4,'linear','extrap'),1);
        b4 = bf;
    end
        
    
    % call to the function f_beta that will compute the intermediary points
    [BB,qq,bbq] = f_beta(p1,p2,p3,p4,b1,b2,b3,b4,Lp,Lb,beta(i),PP,BB,eps);
    
    rk2 = find(PP>=p2,1);
    rk3 = find(PP>=p3,1);
%    CC = 1-BB(rk2:rk3)+(1-PP(rk2:rk3)).*(BB(rk2+1:rk3+1)-BB(rk2-1:rk3-1))./(PP(rk2+1:rk3+1)-PP(rk2-1:rk3-1));
    CC = (1-PP(rk2:rk3)).*(BB(rk2:rk3)-1);

    if ~all(diff(CC)<=0)
        disp('Constraint not satisfied')
        [BB,qq,bbq] = check_constraint(p2,p3,b2,b3,Lp,Lb,beta(i),PP,BB,eps);
    end
    
    Lp = [Lp(:); p2; qq(:)];
    Lb = [Lb(:); b2; bbq(:)];
    
end

Lp = [Lp(:); pp(end)];
Lb = [Lb(:); bb(end)];

%% 1.c. Approximation above the last threshold

BB(PP>=pp(end)) = interp1([Lp(:); 1],[Lb(:); bf],PP(PP>=pp(end)),'pchip','extrap');
BB = max(BB,ones(size(BB))); % ensure that the Pareto coefficients are above 1

%% 1.d. Approximation of the lower part
disp('Estimating the lower part of the Pareto curve...')

% percentiles delimiting the lower part
if isnan(p0)
    p1 = 0;
else
    p1 = p0;
end
p2 = pp(1);

% corresponding Pareto coefficients
if isnan(p0)
    b1 = y_av/thr_min;
else
    b1 = y_av/((1-p0)*thr_min);
end
b2 = bb(1);

% mobile points in the lower part
q1 = 0.9*p1+0.1*p2;
q2 = 0.6*p1+0.4*p2;
% these two mobile points will be adjusted vertically

% value of the Pareto curve at point p given its values bq1 and bq2 at
% mobile points m1 and m2
b_low = @(p,bq1,bq2)(interp1([p1; q1; q2; Lp(:)],[b1; bq1; bq2; Lb(:)],p,'pchip','extrap'));

% function giving the value of the integral between points p1 and p2 as a
% function of the values bq1 and bq2 of the Pareto curve at q1 and q2
int_low = @(bq1,bq2)(quadgk(@(q)(1./((1-q).*b_low(q,bq1,bq2))),p1,p2,'MaxIntervalCount',50000));

% initial guess of bq1 and bq2
bq1_init = max(interp1(pp(1:2),bb(1:2),q1,'linear','extrap'),1);
bq2_init = max(interp1(pp(1:2),bb(1:2),q2,'linear','extrap'),1);

temp_low = int_low(bq1_init,bq2_init);

if temp_low>beta0 % 1st case: if the initial guess of the integral is too high, meaning that the bq1_init and bq2_init are too low
    
    bq1_l = bq1_init;
    bq1_h0 = max(interp1([p1; p2],[b1; b2],q1,'linear','extrap'),bq1_l+1);
    bq1_h = bq1_h0;
    
    bq2_l = bq2_init;
    bq2_h0 = max(interp1([p1; p2],[b1; b2],q2,'linear','extrap'),bq2_l+1);
    bq2_h = bq2_h0;
    
    while int_low(bq1_h,bq2_h)>beta0
        bq1_h = bq1_h+(bq1_h0-bq1_l)/4;
        bq2_h = bq2_h+(bq2_h0-bq2_l)/4;
    end
        
    % adjust bq1 and bq2 while the integral temp_low is not close enough to
    % beta0
    t = cputime;
    while abs(temp_low-beta0)>eps
        
        bq1_temp = (bq1_h+bq1_l)/2;
        bq2_temp = (bq2_h+bq2_l)/2;
        
        if int_low(bq1_temp,bq2_temp)>beta0
            bq1_l = bq1_temp;
            bq2_l = bq2_temp;
        else
            bq1_h = bq1_temp;
            bq2_h = bq2_temp;
        end
        
        temp_low = int_low(bq1_temp,bq2_temp);
        if cputime-t > timelimit
            break
        end
    end
    
    BB(PP<=p2) = b_low(PP(PP<=p2),bq1_temp,bq2_temp);
    
else % 2nd case: if the initial guess of the integral is too low, meaning that the bq1_init and bq2_init are too high
    
    % only one mobile point, q1
    bq1_l = min(b2,bq1_init);
    bq1_h = bq1_init;
    
	% value of the Pareto curve at point p given its values bq1 and bq2 at
	% mobile points m1 and m2
	b_low1 = @(p,bq1)(interp1([p1; q1; Lp(:)],[b1; bq1; Lb(:)],p,'pchip','extrap'));
        
	% function giving the value of the integral between points p1 and p2 as a
	% function of the values bq1 and bq2 of the Pareto curve at q1 and q2
	int_low1 = @(bq1)(quadgk(@(q)(1./((1-q).*b_low1(q,bq1))),p1,p2,'MaxIntervalCount',50000));

    b_min = (b2+p2-p1)/(1+p2-p1);
    if int_low1(b_min)>beta0
    while int_low1(bq1_l)<=beta0
        bq1_l = max(bq1_l-1,1);
    end
    % adjust bq1 while the integral temp_low is not close enough to beta0
    t = cputime;
    while abs(temp_low-beta0)>eps
        
        bq1_temp = (bq1_h+bq1_l)/2;
        
        if int_low1(bq1_temp)>beta0
            bq1_l = bq1_temp;
        else
            bq1_h = bq1_temp;
        end
        
        temp_low = int_low1(bq1_temp);
        
        if cputime-t > timelimit
            break
        end
    end
    
    BB(PP<=p2) = b_low1(PP(PP<=p2),bq1_temp);
    
    else
        b_low_eta = @(p,eta)(((1-p2)*b2-(1+eta)*(p-p2))./(1-p));
        int_low_eta = @(eta)(quadgk(@(q)(1./((1-q).*b_low_eta(q,eta))),p1,p2,'MaxIntervalCount',50000));
        
        eta_l=0;
        eta_h=1;
        
        while int_low_eta(eta_h)>beta0
            eta_h=eta_h+1;
        end
        
        % adjust eta while the integral temp_low is not close enough to
        % beta0
        t = cputime;
        while abs(temp_low-beta0)>eps
            eta_temp = (eta_h+eta_l)/2;
        
            if int_low_eta(eta_temp)>beta0
                eta_l = eta_temp;
            else
                eta_h = eta_temp;
            end
        
            temp_low = int_low_eta(eta_temp);
        
            if cputime-t > timelimit
                break
            end
        end
        
    BB(PP<=p2) = b_low_eta(PP(PP<=p2),eta_temp);
    
    end
end

disp('Pareto curve estimated.')

end

function [BB,qq,bbq] = f_beta(p1,p2,p3,p4,b1,b2,b3,b4,Lp,Lb,beta0,PP,BB,eps)

% minimal separation between mobile percentiles and bracket bounds
sep = 1/5;

% Pareto curve interpolation
b_up = @(p,p2,q1,q2,p3,b2,bq1,bq2,b3,Lp,Lb)(interp1([Lp(:); p2;q1;q2;p3],[Lb(:); b2;bq1;bq2;b3],p,'pchip','extrap'));

% beta-integral between p2 and p3
int_up = @(p2,q1,q2,p3,b2,bq1,bq2,b3,Lp,Lb)(quadgk(@(q)(1./((1-q).*b_up(q,p2,q1,q2,p3,b2,bq1,bq2,b3,Lp,Lb))),p2,p3,'MaxIntervalCount',50000));

% distance between 2 points
distance = @(x1,y1,x2,y2)(((x2-x1).^2+(y2-y1).^2).^0.5);

% time limit for while loops
timelimit = 20;

q1_init = (2/3)*p2+(1/3)*p3;
q2_init = (1/3)*p2+(2/3)*p3;

bq1_init = (2/3)*b2+(1/3)*b3;
bq2_init = (1/3)*b2+(2/3)*b3;

temp = int_up(p2,q1_init,q2_init,p3,b2,bq1_init,bq2_init,b3,Lp,Lb);


if temp>beta0 %1st case: if the initial guess of the integral is too high, meaning that the bq1_init and bq2_init are too low
    
    q1_l = q1_init;
    q1_h = q1_init;
    
    bq1_l = bq1_init;
    bq1_h = bq1_init;
    
    q2_l = q2_init;
    q2_h = q2_init;
    
    bq2_l = bq2_init;
    bq2_h = bq2_init;
    
    % slopes of the successive segments
    d1 = (b2-b1)/(p2-p1);
    d2 = (b3-b2)/(p3-p2);
    d3 = (b4-b3)/(p4-p3);
    
    if (d1>d2)||(d2>d3)
        
        %q1, left
         q1_hl = fzero(@(p)(d2*d1*(p-p1)+d2*b1-(-(p-q1_init)+d2*bq1_init)),0.5);
        bq1_hl = max(d1*(q1_hl-p1)+b1,1);
        
        %q1, right
         q1_hr = fzero(@(p)(d2*d3*(p-p3)+d2*b3-(-(p-q1_init)+d2*bq1_init)),0.5);
        bq1_hr = max(d3*(q1_hr-p3)+b3,1);
        
        %q2, left
         q2_hl = fzero(@(p)(d2*d1*(p-p1)+d2*b1-(-(p-q2_init)+d2*bq2_init)),0.5);
        bq2_hl = max(d1*(q2_hl-p1)+b1,1);
        
        %q2, right
         q2_hr = fzero(@(p)(d2*d3*(p-p3)+d2*b3-(-(p-q2_init)+d2*bq2_init)),0.5);
        bq2_hr = max(d3*(q2_hr-p3)+b3,1);
        
        if (d1>d2)&&(d2>d3)
            if distance(q1_init,bq1_init,q1_hl,bq1_hl)<distance(q1_init,bq1_init,q1_hr,bq1_hr)
                q1_h = q1_hl;
                bq1_h = bq1_hl;
            else
                q1_h = q1_hr;
                bq1_h = bq1_hr;
            end
            if distance(q2_init,bq2_init,q2_hl,bq2_hl)<distance(q2_init,bq2_init,q2_hr,bq2_hr)
                q2_h = q2_hl;
                bq2_h = bq2_hl;
            else
                q2_h = q2_hr;
                bq2_h = bq2_hr;
            end
        elseif d1>d2
            q1_h = q1_hl;
            bq1_h = bq1_hl;
            q2_h = q2_hl;
            bq2_h = bq2_hl;
        else
            q1_h = q1_hr;
            bq1_h = bq1_hr;
            q2_h = q2_hr;
            bq2_h = bq2_hr;
        end
        
        u = -d2/sqrt(1+d2^2);
        v = 1/sqrt(1+d2^2);
        
        vert = 0;
        if (q1_h-p2<=sep*(p3-p2))||(p3-q2_h<=sep*(p3-p2))
            vert=1;
        end
        
        while (int_up(p2,q1_h,q2_h,p3,b2,bq1_h,bq2_h,b3,Lp,Lb)>beta0)&&(q1_h-p2>sep*(p3-p2))&&(p3-q2_h>sep*(p3-p2))
            
             q1_h = q1_h+u;
             q2_h = q2_h+u;
            
            bq1_h = bq1_h+v;
            bq2_h = bq2_h+v;
                        
            if (q1_h-p2<=sep*(p3-p2))||(p3-q2_h<=sep*(p3-p2))
                vert=1;
            end
        end
        
        if vert == 1
            q1_h = (1-sep)*p2+sep*p3;
            q2_h = sep*p2+(1-sep)*p3;
            bq1_h = (1-sep)*b2+sep*b3;
            bq2_h = sep*p2+(1-sep)*b3;
                        
            while int_up(p2,q1_h,q2_h,p3,b2,bq1_h,bq2_h,b3,Lp,Lb)>beta0
                bq1_h = bq1_h+1;
                bq2_h = bq2_h+1;
            end
        end
    else
        
        % (u,v) is a unitary normal vector of the segment of interest
        u = -d2/sqrt(1+d2^2);
        v = 1/sqrt(1+d2^2);
        
        vert = 0;
        if (q1_h-p2<=sep*(p3-p2))||(p3-q2_h<=sep*(p3-p2))
            vert=1;
        end
        
        while (int_up(p2,q1_h,q2_h,p3,b2,bq1_h,bq2_h,b3,Lp,Lb)>beta0)&&(q1_h-p2>sep*(p3-p2))&&(p3-q2_h>sep*(p3-p2))
             q1_h = q1_h+u;
             q2_h = q2_h+u;
            
            bq1_h = bq1_h+v;
            bq2_h = bq2_h+v;
                        
            if (q1_h-p2<=sep*(p3-p2))||(p3-q2_h<=sep*(p3-p2))
                vert=1;
            end
        end
        
        if vert == 1
            q1_h = (1-sep)*p2+sep*p3;
            q2_h = sep*p2+(1-sep)*p3;
            bq1_h = (1-sep)*b2+sep*b3;
            bq2_h = sep*b2+(1-sep)*b3;
            
            while int_up(p2,q1_h,q2_h,p3,b2,bq1_h,bq2_h,b3,Lp,Lb)>beta0
                bq1_h = bq1_h+1;
                bq2_h = bq2_h+1;
            end
        end
    end
    
    % adjust bq1 and bq2 while the integral temp_low is not close enough to
    % beta0
    t = cputime;
    while abs(temp-beta0)>eps
        
         q1_temp = (q1_h+q1_l)/2;
         q2_temp = (q2_h+q2_l)/2;
        
        bq1_temp = (bq1_h+bq1_l)/2;
        bq2_temp = (bq2_h+bq2_l)/2;
        
        if int_up(p2,q1_temp,q2_temp,p3,b2,bq1_temp,bq2_temp,b3,Lp,Lb)>beta0
             q1_l = q1_temp;
             q2_l = q2_temp;
            bq1_l = bq1_temp;
            bq2_l = bq2_temp;
        else
             q1_h = q1_temp;
             q2_h = q2_temp;
            bq1_h = bq1_temp;
            bq2_h = bq2_temp;
        end
        
        temp = int_up(p2,q1_temp,q2_temp,p3,b2,bq1_temp,bq2_temp,b3,Lp,Lb);
        
        if cputime-t > timelimit
            break
        end
    end
    
    BB((PP>=p2)&(PP<=p3)) = b_up(PP((PP>=p2)&(PP<=p3)),p2,q1_temp,q2_temp,p3,b2,bq1_temp,bq2_temp,b3,Lp,Lb);
    qq = [q1_temp; q2_temp];
    bbq = [bq1_temp; bq2_temp];
    
else %2nd case: if the initial guess of the integral is too low, meaning that the bq1_init and bq2_init are too high
    
    q1_l = q1_init;
    q1_h = q1_init;
    
    bq1_l = bq1_init;
    bq1_h = bq1_init;
    
    q2_l = q2_init;
    q2_h = q2_init;
    
    bq2_l = bq2_init;
    bq2_h = bq2_init;
    
    % slopes of the successive segments
    p1
    p2
    d1 = (b2-b1)/(p2-p1)
    d2 = (b3-b2)/(p3-p2)
    d3 = (b4-b3)/(p4-p3)
    
    if (d1<d2)||(d2<d3)
        
        %q1, left
         q1_ll = fzero(@(p)(d2*d1*(p-p1)+d2*b1-(-(p-q1_init)+d2*bq1_init)),0.5);
        bq1_ll = max(d1*(q1_ll-p1)+b1,1);
        
        %q1, right
         q1_lr = fzero(@(p)(d2*d3*(p-p3)+d2*b3-(-(p-q1_init)+d2*bq1_init)),0.5);
        bq1_lr = max(d3*(q1_lr-p3)+b3,1);
        
        %q2, left
         q2_ll = fzero(@(p)(d2*d1*(p-p1)+d2*b1-(-(p-q2_init)+d2*bq2_init)),0.5);
        bq2_ll = max(d1*(q2_ll-p1)+b1,1);
        
        %q2, right
         q2_lr = fzero(@(p)(d2*d3*(p-p3)+d2*b3-(-(p-q2_init)+d2*bq2_init)),0.5);
        bq2_lr = max(d3*(q2_lr-p3)+b3,1);
        
        if (d1<d2)&&(d2<d3)
            if distance(q1_init,bq1_init,q1_ll,bq1_ll)<distance(q1_init,bq1_init,q1_lr,bq1_lr)
                q1_l = q1_ll;
                bq1_l = bq1_ll;
            else
                q1_l = q1_lr;
                bq1_l = bq1_lr;
            end
            if distance(q2_init,bq2_init,q2_ll,bq2_ll)<distance(q2_init,bq2_init,q2_lr,bq2_lr)
                q2_l = q2_ll;
                bq2_l = bq2_ll;
            else
                q2_l = q2_lr;
                bq2_l = bq2_lr;
            end
        elseif d1<d2
            q1_l = q1_ll;
            bq1_l = bq1_ll;
            q2_l = q2_ll;
            bq2_l = bq2_ll;
        else
            q1_l = q1_lr;
            bq1_l = bq1_lr;
            q2_l = q2_lr;
            bq2_l = bq2_lr;
        end
        
        u = -d2/sqrt(1+d2^2);
        v = 1/sqrt(1+d2^2);
        
        vert = 0;
        if (q1_l-p2<=sep*(p3-p2))||(p3-q2_l<=sep*(p3-p2))
        	vert=1;
        end
        
        while (int_up(p2,q1_l,q2_l,p3,b2,bq1_l,bq2_l,b3,Lp,Lb)<=beta0)&&(q1_l-p2>sep*(p3-p2))&&(p3-q2_l>sep*(p3-p2))
             q1_l = q1_l-u;
             q2_l = q2_l-u;
            
            bq1_l = bq1_l-v;
            bq2_l = bq2_l-v;
            
            if (q1_l-p2<=sep*(p3-p2))||(p3-q2_l<=sep*(p3-p2))
                vert=1;
            end
        end
        
        if vert == 1
            q1_l = (1-sep)*p2+sep*p3;
            q2_l = sep*p2+(1-sep)*p3;
            bq1_l = (1-sep)*b2+sep*b3;
            bq2_l = sep*b2+(1-sep)*b3;
            
            while int_up(p2,q1_l,q2_l,p3,b2,bq1_l,bq2_l,b3,Lp,Lb)<=beta0
                bq1_l = bq1_l-1;
                bq2_l = bq2_l-1;
            end
        end
    else
        
        % (u,v) is a unitary normal vector of the segment of interest
        u = -d2/sqrt(1+d2^2);
        v = 1/sqrt(1+d2^2);
        
        vert = 0;
        if (q1_l-p2<=sep*(p3-p2))||(p3-q2_l<=sep*(p3-p2))
        	vert=1;
        end
        
        while (int_up(p2,q1_l,q2_l,p3,b2,bq1_l,bq2_l,b3,Lp,Lb)<=beta0)&&(q1_l-p2>sep*(p3-p2))&&(p3-q2_l>sep*(p3-p2))
             q1_l = q1_l-u;
             q2_l = q2_l-u;
            
            bq1_l = bq1_l-v;
            bq2_l = bq2_l-v;
            
            if (q1_l-p2<=sep*(p3-p2))||(p3-q2_l<=sep*(p3-p2))
                vert=1;
            end
        end
        
        if vert == 1
            q1_l = (1-sep)*p2+sep*p3;
            q2_l = sep*p2+(1-sep)*p3;
            bq1_l = (1-sep)*b2+sep*b3;
            bq2_l = sep*b2+(1-sep)*b3;
            
            while int_up(p2,q1_l,q2_l,p3,b2,bq1_l,bq2_l,b3,Lp,Lb)<=beta0
                bq1_l = bq1_l-1;
                bq2_l = bq2_l-1;
            end
        end
    end
    
    
    % adjust bq1 and bq2 while the integral temp is not close enough to
    % beta0
    t=cputime;
    while abs(temp-beta0)>eps
        
         q1_temp = (q1_h+q1_l)/2;
         q2_temp = (q2_h+q2_l)/2;
        
        bq1_temp = (bq1_h+bq1_l)/2;
        bq2_temp = (bq2_h+bq2_l)/2;
        
        if int_up(p2,q1_temp,q2_temp,p3,b2,bq1_temp,bq2_temp,b3,Lp,Lb)<beta0
             q1_h = q1_temp;
             q2_h = q2_temp;
            bq1_h = bq1_temp;
            bq2_h = bq2_temp;
        else
             q1_l = q1_temp;
             q2_l = q2_temp;
            bq1_l = bq1_temp;
            bq2_l = bq2_temp;
        end
        
        temp = int_up(p2,q1_temp,q2_temp,p3,b2,bq1_temp,bq2_temp,b3,Lp,Lb);
        
        if cputime-t > timelimit
            break
        end

    end
    
    BB((PP>=p2)&(PP<=p3)) = b_up(PP((PP>=p2)&(PP<=p3)),p2,q1_temp,q2_temp,p3,b2,bq1_temp,bq2_temp,b3,Lp,Lb);
    qq = [q1_temp; q2_temp];
    bbq = [bq1_temp; bq2_temp];
    
end

end

function [BB,qq,bbq] = check_constraint(p2,p3,b2,b3,Lp,Lb,beta0,PP,BB,eps)

beta = log((1-p2)/(1-p3))/beta0;

eta = 0.05;

% time limit for while loops
timelimit = 20;

% Pareto curve interpolation
b_chk = @(p,p2,q,p3,b2,bq,b3,Lp,Lb)(interp1([Lp(:); p2; q; p3],[Lb(:); b2; bq; b3],p,'pchip','extrap'));

% beta-integral between p2 and p3
int_chk = @(p2,q,p3,b2,bq,b3,Lp,Lb)(quadgk(@(p)(1./((1-p).*b_chk(p,p2,q,p3,b2,bq,b3,Lp,Lb))),p2,p3,'MaxIntervalCount',50000));

b_min = @(q,p2,b2)(((1-p2)*b2-(1+eta)*(p2-q))./(1-q));

b_max = @(q,p1,b1)(((1-p1)*b1-(1+eta)*(q-p1))./(1-q));

if b2>b3 % 1st case: b is decreasing
    
    if (beta<b2)&&(beta>b3)
        
         q_l = p2;
        bq_l = b3;
    
        x = (p3-p2)^2/((p3-p2)^2+(b3-b2)^2);
         q_h = (1-x)*p3+x*p2;
        bq_h = (1-x)*b3+x*b2;
    
        q_temp = (q_l+q_h)/2;
        bq_temp = (bq_l+bq_h)/2;
    
        temp = int_chk(p2,q_temp,p3,b2,bq_temp,b3,Lp,Lb);
        
        if int_chk(p2,(p2+p3)/2,p3,b2,(b2+b3)/2,b3,Lp,Lb)>beta0
            
            q_h = p3;
            bq_h = b2;
            
            q_l = (1-x)*p2+x*p3;
            bq_l = (1-x)*b2+x*b3;
            
            q_temp = (q_l+q_h)/2;
            bq_temp = (bq_l+bq_h)/2;
    
            temp = int_chk(p2,q_temp,p3,b2,bq_temp,b3,Lp,Lb);

        end
    
        % adjust bq while the integral temp is not close enough to beta0
        t = cputime;
        while abs(temp-beta0)>eps
        
             q_temp = (q_h+q_l)/2;
        
            bq_temp = (bq_h+bq_l)/2;
        
            if int_chk(p2,q_temp,p3,b2,bq_temp,b3,Lp,Lb)>beta0
                 q_l = q_temp;
                bq_l = bq_temp;
            else
                 q_h = q_temp;
                bq_h = bq_temp;
            end
        
            temp = int_chk(p2,q_temp,p3,b2,bq_temp,b3,Lp,Lb);
        
            if cputime-t > timelimit
                break
            end
        end
    
        BB((PP>=p2)&(PP<=p3)) = b_chk(PP((PP>=p2)&(PP<=p3)),p2,q_temp,p3,b2,bq_temp,b3,Lp,Lb);
        qq = [q_temp];
        bbq = [bq_temp];

    elseif beta<=b3
        
        % beta-integral between p2 and p3
        int_chk_decr_min = @(p2,q,p3,b2,b3,Lp,Lb)(quadgk(@(p)(1./((1-p).*b_chk(p,p2,q,p3,b2,b_min(q,p3,b3),b3,Lp,Lb))),p2,q,'MaxIntervalCount',50000)+quadgk(@(p)(1./((1-p).*b_min(p,p3,b3))),q,p3,'MaxIntervalCount',50000));
        
        q_l = p2;
        q_h = p3;
        
        q_temp = (q_l+q_h)/2;
        
        temp = int_chk_decr_min(p2,q_temp,p3,b2,b3,Lp,Lb);
        
        % adjust bq while the integral temp is not close enough to beta0
        t = cputime;
        while abs(temp-beta0)>eps
        
             q_temp = (q_h+q_l)/2;
        
            bq_temp = b_min(q_temp,p3,b3);
        
            if int_chk_decr_min(p2,q_temp,p3,b2,b3,Lp,Lb)>beta0
                 q_l = q_temp;
                bq_l = b_min(q_temp,p3,b3);
            else
                 q_h = q_temp;
                bq_h = b_min(q_temp,p3,b3);
            end
        
            temp = int_chk_decr_min(p2,q_temp,p3,b2,b3,Lp,Lb);
        
            if cputime-t > timelimit
                break
            end
        end
    
        BB((PP>=p2)&(PP<=q_temp)) = b_chk(PP((PP>=p2)&(PP<=q_temp)),p2,q_temp,p3,b2,bq_temp,b3,Lp,Lb);
        BB((PP>=q_temp)&(PP<=p3)) = b_min(PP((PP>=q_temp)&(PP<=p3)),p3,b3);
        qq = [q_temp];
        bbq = [bq_temp];

    else % beta >= b2
        
        % beta-integral between p2 and p3
        int_chk_decr_max = @(p2,q,p3,b2,b3,Lp,Lb)(quadgk(@(p)(1./((1-p).*b_max(p,p2,b2))),p2,q,'MaxIntervalCount',50000)+quadgk(@(p)(1./((1-p).*b_chk(p,p2,q,p3,b2,b_max(q,p2,b2),b3,Lp,Lb))),q,p3,'MaxIntervalCount',50000));
        
        q_l = p2;
        q_h = p3;
        
        q_temp = (q_l+q_h)/2;
        
        temp = int_chk_decr_max(p2,q_temp,p3,b2,b3,Lp,Lb);
        
        % adjust bq while the integral temp is not close enough to beta0
        t = cputime;
        while abs(temp-beta0)>eps
        
             q_temp = (q_h+q_l)/2;
        
            bq_temp = b_max(q_temp,p2,b2);
        
            if int_chk_decr_max(p2,q_temp,p3,b2,b3,Lp,Lb)>beta0
                 q_l = q_temp;
                bq_l = b_max(q_temp,p2,b2);
            else
                 q_h = q_temp;
                bq_h = b_max(q_temp,p2,b2);
            end
        
            temp = int_chk_decr_max(p2,q_temp,p3,b2,b3,Lp,Lb);
        
            if cputime-t > timelimit
                break
            end
        end
    
        BB((PP>=p2)&(PP<=q_temp)) = b_max(PP((PP>=p2)&(PP<=q_temp)),p2,b2);
        BB((PP>=q_temp)&(PP<=p3)) = b_chk(PP((PP>=q_temp)&(PP<=p3)),p2,q_temp,p3,b2,bq_temp,b3,Lp,Lb);
        qq = [q_temp];
        bbq = [bq_temp];
        
    end
    
    
else % 2nd case: b is increasing
    
    c_iii = ((1-1/b2)*log((1-1/b3)/(1-1/b2))+(1/b2)*log(((1-p2)*b2)/((1-p3)*b3))>beta0);
    
    c_iv = ((1-1/b3)*log((1-1/b3)/(1-1/b2))+(1/b3)*log(((1-p2)*b2)/((1-p3)*b3))<beta0);
    
    if (b2<beta)&&(b3>beta)&&c_iii&&c_iv
        
        p_hat2 = 1-(1-p2)*(b2-(1+eta))/(b3-(1+eta));
        p_hat3 = 1-(1-p3)*(b3-(1+eta))/(b2-(1+eta));
        
        b_incr2 = @(p,t,p2,p3,b2,b3,Lp,Lb)(t*b_max(p2+(p_hat2-p2)*(p-p2)/(t*p_hat2+(1-t)*p_hat3-p2),p2,b2)+(1-t)*b2);
        b_incr3 = @(p,t,p2,p3,b2,b3,Lp,Lb)(t*b3+(1-t)*b_min(p3-(p3-p_hat3)*(p3-p)/(p3-t*p_hat2-(1-t)*p_hat3),p3,b3));
        
        int_incr = @(t,p2,p3,b2,b3,Lp,Lb)(quadgk(@(p)(1./((1-p).*b_incr2(p,t,p2,p3,b2,b3,Lp,Lb))),p2,t*p_hat2+(1-t)*p_hat3,'MaxIntervalCount',50000)+quadgk(@(p)(1./((1-p).*b_incr3(p,t,p2,p3,b2,b3,Lp,Lb))),t*p_hat2+(1-t)*p_hat3,p3,'MaxIntervalCount',50000));
        
        t_l = 0;
        t_h = 1;
        
        t_temp = (t_l+t_h)/2;
        
        temp = int_incr(t_temp,p2,p3,b2,b3,Lp,Lb);
        
        % adjust bq while the integral temp is not close enough to beta0
        t = cputime;
        while abs(temp-beta0)>eps
        
             t_temp = (t_h+t_l)/2;
                
            if int_incr(t_temp,p2,p3,b2,b3,Lp,Lb)>beta0
                 t_l = t_temp;
            else
                 t_h = t_temp;
            end
        
            temp = int_incr(t_temp,p2,p3,b2,b3,Lp,Lb);
        
            if cputime-t > timelimit
                break
            end
        end
        
        q_temp = t_temp*p_hat2+(1-t_temp)*p_hat3;
        bq_temp = t_temp*b3+(1-t_temp)*b2;
    
        BB((PP>=p2)&(PP<=q_temp)) = b_incr2(PP((PP>=p2)&(PP<=q_temp)),t_temp,p2,p3,b2,b3,Lp,Lb);
        BB((PP>=q_temp)&(PP<=p3)) = b_incr3(PP((PP>=q_temp)&(PP<=p3)),t_temp,p2,p3,b2,b3,Lp,Lb);
        qq = [q_temp];
        bbq = [bq_temp];
        
    elseif (b2>=beta)||~c_iii
        
        % beta-integral between p2 and p3
        int_chk_incr_min = @(p2,q,p3,b2,b3,Lp,Lb)(quadgk(@(p)(1./((1-p).*b_chk(p,p2,q,p3,b2,b_min(q,p3,b3),b3,Lp,Lb))),p2,q,'MaxIntervalCount',50000)+quadgk(@(p)(1./((1-p).*b_min(p,p3,b3))),q,p3,'MaxIntervalCount',50000));
        
        q_l = p2;
        q_h = p3;
        
        q_temp = (q_l+q_h)/2;
        
        temp = int_chk_incr_min(p2,q_temp,p3,b2,b3,Lp,Lb);
        
        % adjust bq while the integral temp is not close enough to beta0
        t = cputime;
        while abs(temp-beta0)>eps
        
             q_temp = (q_h+q_l)/2;
        
            bq_temp = b_min(q_temp,p3,b3);
        
            if int_chk_incr_min(p2,q_temp,p3,b2,b3,Lp,Lb)>beta0
                 q_l = q_temp;
                bq_l = b_min(q_temp,p3,b3);
            else
                 q_h = q_temp;
                bq_h = b_min(q_temp,p3,b3);
            end
        
            temp = int_chk_incr_min(p2,q_temp,p3,b2,b3,Lp,Lb);
        
            if cputime-t > timelimit
                break
            end
        end
    
        BB((PP>=p2)&(PP<=q_temp)) = b_chk(PP((PP>=p2)&(PP<=q_temp)),p2,q_temp,p3,b2,bq_temp,b3,Lp,Lb);
        BB((PP>=q_temp)&(PP<=p3)) = b_min(PP((PP>=q_temp)&(PP<=p3)),p3,b3);
        qq = [q_temp];
        bbq = [bq_temp];
        
    else
        
        % beta-integral between p2 and p3
        int_chk_incr_max = @(p2,q,p3,b2,b3,Lp,Lb)(quadgk(@(p)(1./((1-p).*b_max(p,p2,b2))),p2,q,'MaxIntervalCount',50000)+quadgk(@(p)(1./((1-p).*b_chk(p,p2,q,p3,b2,b_max(q,p2,b2),b3,Lp,Lb))),q,p3,'MaxIntervalCount',50000));
        
        q_l = p2;
        q_h = p3;
        
        q_temp = (q_l+q_h)/2;
        
        temp = int_chk_incr_max(p2,q_temp,p3,b2,b3,Lp,Lb);
        
        % adjust bq while the integral temp is not close enough to beta0
        t = cputime;
        while abs(temp-beta0)>eps
        
             q_temp = (q_h+q_l)/2;
        
            bq_temp = b_max(q_temp,p2,b2);
        
            if int_chk_incr_max(p2,q_temp,p3,b2,b3,Lp,Lb)>beta0
                 q_l = q_temp;
                bq_l = b_max(q_temp,p2,b2);
            else
                 q_h = q_temp;
                bq_h = b_max(q_temp,p2,b2);
            end
        
            temp = int_chk_incr_max(p2,q_temp,p3,b2,b3,Lp,Lb);
        
            if cputime-t > timelimit
                break
            end
        end
    
        BB((PP>=p2)&(PP<=q_temp)) = b_max(PP((PP>=p2)&(PP<=q_temp)),p2,b2);
        BB((PP>=q_temp)&(PP<=p3)) = b_chk(PP((PP>=q_temp)&(PP<=p3)),p2,q_temp,p3,b2,bq_temp,b3,Lp,Lb);
        qq = [q_temp];
        bbq = [bq_temp];
        
    end
    
end

end
