%% Estimation of the quantile function Q(p) using income tax data
%
% Inputs:
%   * N: vector of percentiles (in [0,1]) where the quantile function has
%   to be estimated
%   * thr: vector of incomes corresponding to the thresholds in the tax
%   tabulation
%   * pp: vector of percentiles (in [0,1]) corresponding to the thresholds in the tax
%   tabulation
%   * bb: vector of (inverted) Pareto coefficients corresponding to the
%   thresholds in the tax tabulation
%   * y_av: average income of the taxpaying population
%   * p0: size of the Dirac in 0
%
% Outputs:
%   * Q: table of approximative values taken by the quantile function Q at
%   the points of table N, that is, Q=Q(N)
%   * B: table of values of the Pareto curve b at the points of table N,
%   that is, B=b(N)
%
% Steps of the program:
%   1. Preliminaries
%   2. Calls function pc.m to estimate the Pareto curve
%   3. Numerically approximates the integral that appears in the quantile function
%   4. Computes a temporary estimation of Q up to a constant
%   5. Adjusts the estimations of Q and clips it to the tax thresholds
%
%% 1. Preliminaries
% Defines function inputs and outputs.

function [Q,B] = quantile(N,thr,pp,bb,y_av,p0)

T = length(thr); % number of thresholds in the tax tabulation
Xn = length(N); % number of points in the input vector N

Ym = 50000000; % number of points where the integrand appearing in the formula giving Q will be calculated

% a and b are the two extremities of the interval where the integrand will
% be calculated
a = min(min(1/Ym,min(N)/2),min(pp)/2);
b = max(max(1-1/Ym,1-(1-max(N))/2),1-(1-max(pp))/2);

% mesh of this interval
nodes = linspace(a,b,Ym+1)';

%% 2. Estimates the Pareto curve
disp('Estimating the Pareto curve...')

% calls function pc to estimate the Pareto curve at the points of nodes
B0 = pc2(nodes,thr,pp,bb,y_av,p0);

% interpolation of the Pareto curve at the points of N
B = qinterp1(nodes,B0,N);
if ~isnan(p0)
    B(N<=p0)=Inf;
end

%% 3. Numerical integration
disp('Numerically approximating the integral that appears in the quantile function...')

% integrand at the points of nodes
integrand = 1./((1-nodes).*B0);

% numerical integration using trapezoidal rule
h = (b-a)/(2*Ym)*[0 ones(1,2)];
trapz = conv(integrand, h, 'same');
int = cumsum(trapz);

clear trapz

%% 4. Computes a temporary estimation of Q up to a constant
disp('Computing a temporary estimation of Q up to a constant...')

% Q up to a constant estimated at the points of nodes
temp0 = integrand.*exp(-int);

clear int integrand

% Q up to a constant interpolated at the points of N
temp = qinterp1(nodes,temp0,N);

% values of temp at the raw data thresholds
q0 = qinterp1(nodes,temp0,pp);

% multiplying factors at the raw data thresholds
% the elements of Q_ast should all be approximately equal
Q_ast = thr./q0;

clear nodes temp0

%% 5. Adjusts the estimations of Q and clips it to the tax thresholds
disp('Adjusting the estimations of Q and clips it to the tax thresholds...')

% vector which will contain the numerical approximation of the final 
% quantile function Q
Q = zeros(Xn,1);

% within each bracket, the final value Qfin is a weighted average of the
% quantile functions starting from the two thresholds bounding the bracket

[br,~] = bracket(pp,N); % if p_i<=N_i<p_i+1, then br(N_i)=i

% if before the first threshold, use the quantile function starting from the 
% first threshold
Q(br(:,1)==0) = Q_ast(1).*temp(br(:,1)==0);

% below p0, the quantile function is equal to 0
if ~isnan(p0)
	Q(N<=p0)=0;
end

% if after the last threshold, use the quantile function starting from the 
% last threshold
Q(br(:)==T) = Q_ast(T)*temp(br(:)==T);

% else, use a weighted average of the two surrounding thresholds
% vectors of bracketing thresolds
Q1 = zeros(Xn,1);
Q2 = zeros(Xn,1);

Q1(min(br(:,1)>=1,br(:,1)<T)) = Q_ast(br(min(br(:,1)>=1,br(:,1)<T)),1); % threshold below
Q2(min(br(:,1)>=1,br(:,1)<T)) = Q_ast(br(min(br(:,1)>=1,br(:,1)<T))+1,1); % threshold above

% weights to be applied to the quantile function starting at p_i
w1 = zeros(Xn,1);
w1(min(br(:)>=1,br(:)<T)) = (pp(br(min(br(:,1)>=1,br(:,1)<T))+1)-N(min(br(:,1)>=1,br(:,1)<T)))./(pp(br(min(br(:,1)>=1,br(:,1)<T))+1)-pp(br(min(br(:,1)>=1,br(:,1)<T))));

% weights to be applied to the quantile function starting at p_i+1
w2 = zeros(Xn,1);
w2(min(br(:,1)>=1,br(:,1)<T)) = (N(min(br(:,1)>=1,br(:,1)<T))-pp(br(min(br(:,1)>=1,br(:,1)<T))))./(pp(br(min(br(:,1)>=1,br(:,1)<T))+1)-pp(br(min(br(:,1)>=1,br(:,1)<T))));

% quantile function between p1 and pT
Q(min(br(:,1)>=1,br(:,1)<T)) = (w1(min(br(:)>=1,br(:)<T)).*Q1(min(br(:)>=1,br(:)<T))+w2(min(br(:)>=1,br(:)<T)).*Q2(min(br(:)>=1,br(:)<T))).*temp(min(br(:)>=1,br(:)<T));


disp('Quantile function estimated.')
