%% Estimation of the generalized Pareto curve b(p) using income tax data
%
% Method: PCHIP interpolation
%
% Inputs:
%   * pp: table of percentiles (in [0,1]) corresponding to the thresholds in the tax
%   tabulation
%   * bb: table of (inverted) Pareto coefficients corresponding to the
%   thresholds in the tax tabulation
%   * P: table of points p (in [0,1]) where b(p) has to be calculated
%
% Output:
%   * B: table of approximative values taken by b at points of the input
%   table P, that is, B=b(P)
%
% Steps of the program:
%   1. Preliminaries
%   2. Interpolation of the Pareto curve above the lower threshold using
%   PCHIP method
%
%% 1. Preliminaries
% Defines function inputs and outputs.

function [B,pp_mid,bb_out] = pc2(pp,bb,beta,P)

%% 2. Interpolation
mid = (pp(1:(end-1))+pp(2:end))/2;
quart1 = (pp(1:(end-1))+mid)/2;
quart2 = (pp(1:(end-1))+mid)/2;

%pp_mid = [pp; mid];
pp_mid = [reshape([pp(1:end-1) quart1 mid quart2],[],1); pp(end)]

f = @(p,bm_quart1,bm_mid,bm_quart2)(interp1(pp_mid,[reshape([bb(1:end-1) bm_quart1 bm_mid bm_quart2],[],1); bb(end)],p,'pchip','extrap'));

f_beta = @(p,bm_quart1,bm_mid,bm_quart2)(integral(@(q)(1./((1-q).*f(q,bm_quart1,bm_mid,bm_quart2))),pp(1),p));
bm_quart1_temp = interp1(pp,bb,quart1);
bm_quart2_temp = interp1(pp,bb,quart2);
bm_mid_temp = interp1(pp,bb,mid);

%f_beta(bb(2),bm_temp)
%fmin = @(bm)(sum((f_beta(bb(2:end),bm)-beta).^2));

fmin = @(bm_quart1,bm_mid,bm_quart2)((f_beta(pp(2),bm_quart1,bm_mid,bm_quart2)-beta(1)).^2);
for i=3:length(bb)
    fmin = @(bm_quart1,bm_mid,bm_quart2)(fmin(bm_quart1,bm_mid,bm_quart2)+(f_beta(pp(i),bm_quart1,bm_mid,bm_quart2)-f_beta(pp(i-1),bm_quart1,bm_mid,bm_quart2)-beta(i-1)).^2);
end
fmin(bm_quart1_temp,bm_mid_temp,bm_quart2_temp)

max_bm = [];%(bb(1:(end-1))+bb(2:end))/2;

options = optimoptions('fmincon','Display','off');
bb_mid = fmincon(@(bm)fmin(bm),[bm_quart1_temp;bm_mid_temp;bm_quart2_temp],[],[],[],[],ones(length(bb)-1,1),[],[],options);

bb_opt = reshape(bb_mid,[],3);
fmin(bb_opt(:,1),bb_opt(:,2),bb_opt(:,3))

B = interp1(pp_mid,[reshape([bb(1:end-1) bb_opt(:,1) bb_opt(:,2) bb_opt(:,3)],[],1); bb(end)],P,'pchip','extrap');

B(P<pp(1,1)-0.05) = 1;

%exp(f_beta(pp(end),bb_mid)-beta(end))
exp(-f_beta(pp(end),bm_quart1_temp,bm_mid_temp,bm_quart2_temp)+sum(beta))

exp(-f_beta(pp(end),bb_opt(:,1),bb_opt(:,2),bb_opt(:,3))+sum(beta))

%exp(-f_beta(pp(end),bb_mid)-beta(end))
%exp(f_beta(pp(end),bb_mid)+beta(end))

pp_draw = linspace(0,1,1000);
bb_draw = qinterp1(P,B,pp_draw);
plot(pp_draw,bb_draw)
hold on
scatter(pp,bb)

bb_out = reshape([bb(1:end-1) bb_opt(:,1) bb_opt(:,2) bb_opt(:,3)]);

disp('Pareto curve estimated.')