%% Estimation of the generalized Pareto curve b(p) using income tax data
%
% Method: PCHIP interpolation
% This routine extrapolates the lower part of the distribution.
%
% Inputs: 
%   * thr: table of incomes corresponding to the thresholds in the tax
%   tabulation
%   * pp: table of percentiles (in [0,1]) corresponding to the thresholds in the tax
%   tabulation
%   * bb: table of (inverted) Pareto coefficients corresponding to the
%   thresholds in the tax tabulation
%   * y_av: average income of the taxpaying population
%   * P: table of points p (in [0,1]) where b(p) has to be calculated
%
% Outputs:
%   * B: table of approximative values taken by b at points of the input
%   table P, that is, B=b(P)
%   * p0: estimation of the share of the population with no income
%
% Steps of the program:
%   1. Preliminaries
%   2. Interpolation of the Pareto curve above the lower threshold using
%   PCHIP method
%   3. Extrapolation of the Pareto curve below the lower threshold
%   4. Merges the tables B_interp and B_extrap
%   5. Plots the Pareto curve
%
%% 1. Preliminaries
% Defines function inputs and outputs.

function [B,p0] = pc(thr,pp,bb,y_av,P)

%% 2. Interpolation

interp_b = pchip(pp,bb);

%B_interp = ppval(interp_b,P);
B = ppval(interp_b,P);
p0 = pp(1,1);

i=1;
while P(i,1)<=p0
    B(i,1) = 1;
    i=i+1;
end

% %% 3. Extrapolation of b(p) for lower incomes
% 
% p1 = pp(1,1);
% t1 = thr(1,1);
% b1 = bb(1,1);
% 
% l1 = 1-((1-p1)*b1*t1)/y_av;
% l1_pr = (1-l1)/((1-p1)*b1);
% 
% p0=p1-2*(l1/l1_pr);
% 
% syms x
% x1 = (x-p0)/(p1-p0);
% x2 = (x-p0)/(p1-p0);
% 
% syms y
% phi = 3*y^2-2*y^3;
% psi = y^3-y^2;
% 
% syms z
% L_pol = l1*compose(phi,x1,y,x,z)+l1_pr*(p1-p0)*compose(psi,x2,y,x,z);
% L = sym2poly(L_pol);
% 
% L_pr = polyder(L);
% 
% B_extrap = (1-polyval(L,P))./((1-P).*(polyval(L_pr,P)));
% 
% if p0<0
%     p0=0;
% end

%% 4. Merges the tables B_interp and B_extrap
% B = B_interp;
% i=1;
% 
% while P(i,1)<=p0
%     B(i,1) = 1;
%     i=i+1;
% end
% 
% while P(i,1)<=p1
%     B(i,1) = B_extrap(i,1);
%     i=i+1;
% end

disp('Pareto curve estimated')

%% 5. Plots the Pareto curve

xx = linspace(0,1,1000);
yy = interp1(P,B,xx,'linear');

plot(xx,yy)
hold on
scatter(pp,bb)
title('Interpolated Pareto curve');